/*****************************************************************************
 ** Class QSubtitles
 **
 ** This class handles the subtitles - tab of the main QDVDAuthor 
 ** application
 **
 **   Created : 
 **        by : Varol Okan
 ** Copyright : (c) Varol Okan
 ** License   : GPL v 2.0
 **
 *****************************************************************************/

#ifndef SUBTITLESGUI_H
#define SUBTITLESGUI_H

#include <qlabel.h>
//#include <q3dragobject.h> //ooo
//Added by qt3to4:
#include <QDropEvent>
//#include <Q3GridLayout>   //ooo
#include <QGridLayout>   //xxx
#include <QMouseEvent>
#include <QEvent>
#include <QDragEnterEvent>
#include <QScrollArea>  //xxx

//#include "uisubtitles.h"	//ooo
#include "ui_uisubtitles.h"	//xxx
#include "rgba.h"

class Subtitles;
class QDVDAuthor;
//class Q3GridLayout;   //ooo
//class Q3ScrollView;   //ooo
class SubtitlesGui;
class MediaInterface;
class SourceFileInfo;
class SourceFileEntry;

class MediaLabel : public QLabel
{
  Q_OBJECT
public:
           //MediaLabel ( SubtitlesGui *, QWidget *p, const char *n=NULL, Qt::WFlags f=0 );
           MediaLabel ( SubtitlesGui *, QWidget *p, const char *n=NULL, Qt::WindowFlags f=Qt::WindowFlags() );
  virtual ~MediaLabel ( );

protected:
   //virtual void dropEvent      ( QDropEvent * );  //oooo
   //virtual void dragEnterEvent ( QDragEnterEvent * ); //oooo

private:
   SubtitlesGui *m_pSubtitlesGui;
};

class PreviewLabel : public QLabel
{
  Q_OBJECT
public:
           //PreviewLabel ( SubtitlesGui *, QWidget *p, const char *n=NULL, Qt::WFlags f=0 );   //ooo
           PreviewLabel ( SubtitlesGui *, QWidget *p, const char *n=NULL, Qt::WindowFlags f=Qt::WindowFlags() ); //xxx
  virtual ~PreviewLabel ( );

protected:
   virtual void mouseDoubleClickEvent ( QMouseEvent * );
   virtual void mouseMoveEvent        ( QMouseEvent * );
   virtual void mousePressEvent       ( QMouseEvent * );

private:
   SubtitlesGui *m_pSubtitlesGui;
   QPoint        m_lastPos;
};

//class SubtitlesGui : public uiSubtitles			//ooo
class SubtitlesGui : public QWidget, public Ui::uiSubtitles	//xxx
{
  Q_OBJECT

  enum enPreviewMode { ModeFitWindow=0, ModeHalfScroll, ModeScroll };
public:
   //SubtitlesGui ( QDVDAuthor *, QWidget *p, const char * n=0, Qt::WFlags f=0 );	//ooo
   SubtitlesGui ( QDVDAuthor *pDVDauthor = 0, QWidget *pParent = 0 );			//xxx
  ~SubtitlesGui ( );

public slots:
  virtual void slotNewPosition          ( int, const QString & );
  virtual void slotNewPosition          ( long );
  virtual void slotCloseSource          ( );
  virtual void slotMediaFB              ( );
  virtual void slotMediaBack            ( );
  virtual void slotMediaStop            ( );
  virtual void slotMediaPause           ( );
  virtual void slotMediaPlay            ( );
  virtual void slotMediaPlayStop        ( );
  virtual void slotMediaFF              ( );
  virtual void slotAddNew               ( );
  virtual void slotStart                ( );
  virtual void slotEnd                  ( );
  virtual void slotImport               ( );
  virtual void slotExport               ( );
  virtual void slotClear                ( );
  virtual void slotChange               ( );
  virtual void slotDelete               ( );
  virtual void slotDeleteLine           ( );
  virtual void slotSearchText           ( );
  virtual void slotApplyToAll           ( );
  virtual void slotFontColor            ( );
  virtual void slotAddSubtitle          ( );
  virtual void slotBackgroundColor      ( );
  virtual void slotRenderPreview        ( );
  virtual void slotFontFamilyChanged    ( int  );
  virtual void slotFontStyleChanged     ( int  );
  virtual void slotFontSizeChanged      ( int  );
  virtual void slotVolumeChanged        ( int  );
  virtual void slotPositionChanged      ( int  );
  virtual void slotTransparencyChanged  ( int  );
  virtual void slotBGTransparencyChanged( int  );
  virtual void slotTextOnly             ( bool );
  virtual void slotRight                ( bool );
  virtual void slotCenter               ( bool );
  virtual void slotLeft                 ( bool );
  virtual void slotJustify              ( bool );
  virtual void slotVTop                 ( bool );
  virtual void slotVCenter              ( bool );
  virtual void slotVBottom              ( bool );
  virtual void slotFit                  ( bool );
  virtual void slotUnderline            ( bool );
  virtual void slotOutline              ( bool );
  virtual void slotTabChanged           ( QWidget * );
  //virtual void slotSubtitleChanged      ( Q3ListViewItem * ); //ooo
  virtual void slotSubtitleChanged      ( QTreeWidgetItem * );    //xxx

protected slots:
  virtual bool event      ( QEvent * );
  virtual void slotSetSelectedItem ( );

public:
  //void insertDraggedObject    ( QDropEvent *, QImage  & );    //oooo
  //void insertDraggedText      ( QDropEvent *, QString & );    //oooo
  Subtitles *getSubtitles     ( bool b=false );
  void releaseSourceFileInfo  ( );
  void createTableEntries     ( );
  void previewDoubleClick     ( );
  void scrollPreview ( int, int );

private:
  void initMe                                       ( );
  void displayRow                              (  int );
  void updateFont                              ( uint );
  bool addSubtitle                          ( QString );
  int  findString            ( QStringList &, QString );
  bool createSubtitles                              ( );
  void updateButtons                                ( );
  void resetMediaWidget                             ( );
  void renderPreview                                ( );
  QFont &getFont                                    ( );
  bool getSubtitleRect                    ( QRect   & );
  void setBackgroundColor                 ( Rgba    & );
  void setForegroundColor                 ( Rgba    & );
  void initNewSourceFileInfo              (      bool );
  bool sanityCheckSubtitleTrack           (     int & );
  SourceFileInfo  *findSourceFileInfo     ( QString & );
  SourceFileEntry *sourceFileEntry ( SourceFileInfo * );
  void displayPreview              ( QImage &,QRect & );

private:
  MediaLabel     *m_pDragLabel;
  //Q3GridLayout    *m_pLayout; //ooo
  QGridLayout    *m_pLayout;   //xxx
  MediaInterface *m_pMediaInterface;
  //Q3ScrollView    *m_pScrollView; //ooo
  QScrollArea    *m_pScrollView;   //xxx
  QLabel         *m_pPreview;
  SourceFileInfo *m_pSourceFileInfo;
  Subtitles      *m_pSubtitles; // separated from the actual Subtitle in the SourceFileInfo.
  QDVDAuthor     *m_pDVDAuthor;
  QString         m_qsCurrentURL;
  QString         m_strCurrentTime;
  QString         m_strStartTime;
  QString         m_strEndTime;
  QFont           m_qfFont;
  int             m_iAlignement;  //ooo
  //Q_DECLARE_FLAGS   m_iAlignement;    //xxx
  Rgba            m_backgroundColor;
  Rgba            m_foregroundColor;
  int             m_iCurrentSubtitleNumber;
  enPreviewMode   m_enPreviewMode;
  //Q3ListViewItem  *m_pSelectItem; //ooo
  QTreeWidgetItem  *m_pSelectItem;   //xxx
  int             m_iOrigSubtitlesState;
  int             m_iVideoHeight;
  int             m_iVideoWidth;
};

#endif // SUBTITLESGUI_H
